/*
 * HSPMap.cpp
 *
 *  Created on: Mar 24, 2010
 *      Author: Oleksiy Kohany <kohany@gmail.com>
 */

#include "HSPMap.h"

namespace bio {

HSPMap::HSPMap(bool top_p, bool bot_p) : top_p(top_p), bot_p(bot_p) {
}

HSPMap::~HSPMap() {
}

void HSPMap::sort() {
   hspmap_t::iterator rmptr;
   for (rmptr = _map.begin(); rmptr != _map.end(); rmptr++) {
      (*rmptr).second.sort();
   }
}
//*******************************************
void HSPMap::write(ostream &os) const {
   const_iterator rmptr;
   hsplist_t::const_iterator lptr;
   //sort();
   for (rmptr = _map.begin(); rmptr != _map.end(); rmptr++) {
      for (lptr = (*rmptr).second.begin(); lptr != (*rmptr).second.end(); lptr++) {
         HSP hsp = *lptr;
         if (top_p ^ bot_p) {
            if (top_p) {
               hsp.getTop() /= 3;
            }
            if (bot_p) {
               hsp.getBot() /= 3;
            }
         }
         os << hsp << endl;
      }
   }
}

HSPMap& HSPMap::read(istream& is, bool noSelf) {
   _map.clear();
   bool needsSorting = false;
   size_t lineNo = 0;
   HSP hsp;
   HSPReader reader;
   //cerr << "Reading...";
   while (!is.eof()) {
      ++lineNo;
      try {
         reader.readHSP(is, hsp);
      } catch (istream::failure e) {
         if(!is.eof()) {
            cerr << "ERROR: Failed to read line #" << lineNo << endl;
            cerr << e.what() << endl;
         }
         continue;
      }
      if (noSelf && hsp.getTop() == hsp.getBot()) {
         continue;
      }
      if (top_p ^ bot_p) {
         if (top_p) {
            hsp.getTop() *= 3;
         }
         if (bot_p) {
            hsp.getBot() *= 3;
         }
      }
      if (!needsSorting) {
         iterator mptr = _map.find(hsp.getTop().getName());
         if (mptr != _map.end() && (*mptr).second.back() > hsp) {
            needsSorting = true;
         }
      }
      _map[hsp.getTop().getName()].push_back(hsp);
   }
   //cerr << "done" << endl;
   HSP::setCmpBy( TOPBEG);
   if (needsSorting) {
      sort();
   }
   return *this;
}

istream& operator >>(istream& is, HSPMap& mp) {
   mp.read(is);
   return is;
}
ostream& operator <<(ostream& os, const HSPMap& mp) {
   mp.write(os);
   return os;
}
};
